CREATE TABLE IF NOT EXISTS `{table}`
	(`player_uuid` CHAR(36) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
	`id` VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
	`earn_date` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
	UNIQUE `player_uuid_index` (`player_uuid`, `id`) USING BTREE);

-- Create stored procedure used to loop through passed achievements
DROP procedure IF EXISTS `mbedwars_player_achievements_update`;

CREATE PROCEDURE `mbedwars_player_achievements_update`
	(_player_uuid CHAR(36) CHARACTER SET ascii COLLATE ascii_bin,
	_raw_achievements MEDIUMTEXT)
BEGIN
	-- init everything
    DECLARE _next TEXT DEFAULT NULL;
    DECLARE _nextlen INT DEFAULT NULL;
    DECLARE _middlei INT DEFAULT NULL;
    DECLARE _id TEXT DEFAULT NULL;
    DECLARE _date TEXT DEFAULT NULL;
    DECLARE _index INT DEFAULT 0;
    DECLARE _length INT DEFAULT 0;
	
	CREATE TEMPORARY TABLE _earned_achievements (`id` varchar(255), `earn_date` timestamp) ENGINE=MEMORY;
	INSERT INTO _earned_achievements SELECT `id`, `earn_date` FROM `{table}` WHERE `player_uuid` = _player_uuid;

    -- first go through all achievements
    iterator:
    LOOP
    	-- exit the loop if the list seems empty or was null;
        -- this extra caution is necessary to avoid an endless loop in the proc.
        IF LENGTH(TRIM(_raw_achievements)) = 0 OR _raw_achievements IS NULL THEN
        	LEAVE iterator;
        END IF;
        
        -- capture the next value from the list
        SET _next = SUBSTRING_INDEX(_raw_achievements,',',1);
        
        -- save the length of the captured value; we will need to remove this
        -- many characters + 1 from the beginning of the string 
        -- before the next iteration
        SET _nextlen = LENGTH(_next);
        
        -- parse the entry
        SET _id = SUBSTRING_INDEX(_next, '!', 1);
        SET _date = STR_TO_DATE(INSERT(_next, 1, LENGTH(_id)+1, ''), '%m-%d-%Y %H:%i:%s');
        
        -- we haven't earned it yet. add it
        IF ((SELECT COUNT(*) FROM _earned_achievements WHERE `id` = _id) = 0) THEN
            INSERT INTO `{table}` (`player_uuid`, `id`, `earn_date`) VALUES (_player_uuid, _id, _date);
        	
        -- we already earned it, so remove it from the temp table
        -- we'll go through the table after this iterator and remove them from the actual table
        ELSE
        	-- update the earn date if it has changed
            IF ((SELECT COUNT(*) FROM _earned_achievements WHERE `id` = _id AND `earn_date` = _date) = 0) THEN
            	UPDATE `{table}` SET `earn_date`=_date WHERE `player_uuid` = _player_uuid AND `id` = _id;
            END IF;
            
        	DELETE FROM _earned_achievements WHERE `id` = _id;
        END IF;
        
        -- rewrite the original string using the `INSERT()` string function,
        -- args are original string, start position, how many characters to remove, 
        -- and what to "insert" in their place (in this case, we "insert"
        -- an empty string, which removes _nextlen + 1 characters)
        SET _raw_achievements = INSERT(_raw_achievements, 1, _nextlen + 1,'');
    END LOOP;
    
    -- go through all unearned achievements
    SELECT COUNT(*) FROM _earned_achievements INTO _length;
    
    iterator:
    LOOP
    	-- exit loop if we reached the end
        IF _index = _length THEN
        	LEAVE iterator;
        END IF;
        
        -- remove the achievement that we unearned
    	DELETE FROM `{table}` WHERE `player_uuid` = _player_uuid AND `id` = (SELECT @entry := `id` FROM _earned_achievements LIMIT _index,1);
        
        SET _index = _index + 1;
    END LOOP;
    
    -- delete temp table as it otherwise doesn't get removed since connections are in a pool
    DROP TABLE _earned_achievements;
END;