-- Create table
CREATE TABLE IF NOT EXISTS `{table}`
	(`player_uuid` CHAR(36) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
	`id` VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
	`value` VARCHAR(8192) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
	UNIQUE `index_base` (`player_uuid`, `id`) USING BTREE);
	
-- Create stored procedure used to loop through passed properties
DROP procedure IF EXISTS `mbedwars_player_properties_update`;

CREATE PROCEDURE `mbedwars_player_properties_update`
	(_player_uuid CHAR(36) CHARACTER SET ascii COLLATE ascii_bin,
	_raw_properties MEDIUMTEXT CHARACTER SET ascii COLLATE ascii_bin)
BEGIN
	-- init everything
    DECLARE _next TEXT CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL;
    DECLARE _nextlen INT DEFAULT NULL;
    DECLARE _middlei INT DEFAULT NULL;
    DECLARE _id TEXT DEFAULT NULL;
    DECLARE _value TEXT CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL;
    DECLARE _index INT DEFAULT 0;
    DECLARE _length INT DEFAULT 0;
    
    -- first go through all properties
    iterator:
    LOOP
    	-- exit the loop if the list seems empty or was null;
        -- this extra caution is necessary to avoid an endless loop in the proc.
        IF LENGTH(TRIM(_raw_properties)) = 0 OR _raw_properties IS NULL THEN
        	LEAVE iterator;
        END IF;
        
        -- capture the next value from the list
        SET _next = SUBSTRING_INDEX(_raw_properties,',',1);
        
        -- save the length of the captured value; we will need to remove this
        -- many characters + 1 from the beginning of the string 
        -- before the next iteration
        SET _nextlen = LENGTH(_next);
        
        -- parse the entry
        SET _id = SUBSTRING_INDEX(_next, '!', 1);
        SET _value = FROM_BASE64(INSERT(_next, 1, LENGTH(_id)+1, ''));
        
        -- update property
        REPLACE INTO `{table}` (`player_uuid`, `id`, `value`) VALUES (_player_uuid, _id, _value);
        
        -- rewrite the original string using the `INSERT()` string function,
        -- args are original string, start position, how many characters to remove, 
        -- and what to "insert" in their place (in this case, we "insert"
        -- an empty string, which removes _nextlen + 1 characters)
        SET _raw_properties = INSERT(_raw_properties, 1, _nextlen + 1,'');
    END LOOP;
END;